indexing

	description: "Do StartServiceCtrlDispatcher in a separate thread, thanks to Delphi 5 for the idea. StartServiceCtrlDispatcher itself does not return and the handler functions want to execute in the primary thread."


class

	SERVICE_THREAD


inherit

	THREAD
		rename
			thread_id as thread_handle
		end

	SERVICE_DEBUGGING
		export
			{NONE} all
		end


creation

	make


feature {NONE} -- Initialization

	make (a_service: like service) is
		require
			service_not_void: a_service /= Void
		do
			service := a_service
			create msg.make
		end


feature -- Access

	thread_id: INTEGER

	service: SERVICE
			-- The service for which this thread is executing.


feature -- Execution

	execute is
		do
			-- Create a message queue so we can receive the requests send
			-- by SERVICE.controller
			thread_id := get_current_thread_id
			msg.create_thread_message_queue
			service.execute
		end

	process_requests (wait_for_message: BOOLEAN) is
		local
			action: INTEGER
		do
			debug ("service")
				log ("SERVICE_THREAD.process_requests begin")
			end
			if not terminated or else wait_for_message then
				from
					if wait_for_message then
						msg.get_all
					else
						msg.peek_all
					end
				until
					terminated or else
					not msg.last_boolean_result
				loop
					if msg.is_thread_message then
						if msg.message = service.cm_service_control_code then
							-- protect against exceptions here
							action := msg.wparam
							inspect action
							when service_control_stop then
								service.stop
							when service_control_shutdown then
								service.shutdown
							when service_control_interrogate then
								service.interrogate
							else
								--ignore other actions
							end
						else
							msg.dispatch
						end
					else
						msg.dispatch
					end
					if not terminated and then wait_for_message then
						mssleep (1000)
						msg.get_all
					end
				end
			end
			debug ("service")
				log ("SERVICE_THREAD.process_requests end")
			end
		rescue
			debug ("service")
				log ("SERVICE_THREAD.process_requests rescue")
			end
		end

	terminate is
		do
			terminated := True
		end


feature {NONE} -- Implementation

	msg: X_WEL_MSG
			-- Access to thread message queue.


feature {NONE} -- win32 API

	get_current_thread_id: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			"GetCurrentThreadId()"
		end


feature {NONE} -- Service constants

	service_control_stop: INTEGER is 1
	service_control_pause: INTEGER is 2
	service_control_continue: INTEGER is 3
	service_control_interrogate: INTEGER is 4
	service_control_shutdown: INTEGER is 5


invariant

	msg_not_void: msg /= Void
	service_not_void: service /= Void

end
